package main

import (
	_ "image/jpeg"
    "fmt"
    "image/color"
    "io/ioutil"
    "log"
    "strings"
    "github.com/golang/freetype/truetype"
    "golang.org/x/image/font"

    "github.com/hajimehoshi/ebiten"
    "github.com/hajimehoshi/ebiten/ebitenutil"
    "github.com/hajimehoshi/ebiten/text"
    "github.com/hajimehoshi/ebiten/audio"
    "github.com/hajimehoshi/ebiten/audio/mp3"
)

const (
    screenWidth  = 640
    screenHeight = 480
    sampleRate   = 44100
)

type dialogue struct {
	Speaker character
	Text string
}

type character struct {
	Name string
	Color color.RGBA
}

var (
	m = character{`Me`, color.RGBA{200, 200, 255, 255}}
	s = character{`Sylvie`, color.RGBA{200, 255, 200, 255}}
	d = character{``, color.RGBA{0, 255, 0, 0}}
	messages = []dialogue{
        {d, `playSong ./Departures.mp3`},
		{s, `Hi there! How was class?`},
		{m, `Good...`}, 
		{d, `I can't bring myself to admit that it all went in one ear and out the other.`},
        {s, `Are you going home now? Wanna walk back with me?`},
        {d, `playSong ./Hisoku.mp3`},
        {m, `Sure!`},
	}
    mplusNormalFont font.Face
    line = 0
    bg *ebiten.Image
    audioContext *audio.Context
    audioPlayer  *audio.Player
    playingMusic = false
)

func playSong(path string) {
    var err error
    if playingMusic {
        audioPlayer.Close()
    }
    f, err := ebitenutil.OpenFile(path)
    if err != nil {
        log.Fatal(err)
    }
    d, err := mp3.Decode(audioContext, f)
    if err != nil {
        log.Fatal(err)
    }
    audioPlayer, err = audio.NewPlayer(audioContext, d)
    if err != nil {
        log.Fatal(err)
    }
    audioPlayer.Play()
    playingMusic = true
}

func init() {

    f, err := ebitenutil.OpenFile("./mplus-1p-regular.ttf")
    if err != nil {
        log.Fatal(err)
    }
    defer f.Close()

    b, err := ioutil.ReadAll(f)
    if err != nil {
        log.Fatal(err)
    }

    tt, err := truetype.Parse(b)
    if err != nil {
        log.Fatal(err)
    }

    const dpi = 72
    mplusNormalFont = truetype.NewFace(tt, &truetype.Options{
        Size:    24,
        DPI:     dpi,
        Hinting: font.HintingFull,
    })

    audioContext, err = audio.NewContext(sampleRate)
    if err != nil {
        log.Fatal(err)
    }
}


func update(screen *ebiten.Image) error {

    if ebiten.IsRunningSlowly() {
        return nil
    }
    action := strings.Split(messages[line].Text, " ")
    for _, v := range ebiten.InputChars() {
    	if v == 32 {
            if line+1 != len(messages) {
                line++
            }
    	}
    }
    action = strings.Split(messages[line].Text, " ")
    if (action[0] == "playSong") {
        if line+1 != len(messages) {
            playSong(action[1])
            line++
        }
    }

    msg := fmt.Sprintf("FPS: %0.2f", ebiten.CurrentFPS())
    const x = 20

    op := &ebiten.DrawImageOptions{}

    screen.DrawImage(bg, op)

    ebitenutil.DrawRect(screen, 30, 350, screenWidth-60, 100, color.RGBA{128, 128, 128, 200})

    text.Draw(screen, msg, mplusNormalFont, x, 40, color.White)
    text.Draw(screen, messages[line].Speaker.Name, mplusNormalFont, 50, 370, messages[line].Speaker.Color)
    text.Draw(screen, messages[line].Text, mplusNormalFont, 50, 400, color.White)
    return nil
}

func main() {
	var err error
	bg, _, err = ebitenutil.NewImageFromFile("./bg.jpg", ebiten.FilterNearest)
    if err != nil {
        log.Fatal(err)
    }
    if err := ebiten.Run(update, screenWidth, screenHeight, 1, "Font (Ebiten Demo)"); err != nil {
        log.Fatal(err)
    }
}